import 'dart:math';

import 'package:dimension/dimension.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:morphable_shape/morphable_shape.dart';
import 'package:responsive_styled_widget/styled_widget.dart';

class ListViewPage extends StatefulWidget {
  ListViewPage({this.title = "Transform"});

  final String title;

  @override
  _ListViewPageState createState() => _ListViewPageState();
}

class _ListViewPageState extends State<ListViewPage> {
  bool toggleStyle = true;

  late Style beginStyle;
  late Map<ScreenScope, Style> endStyle;

  @override
  void initState() {
    super.initState();

    beginStyle = Style(
      alignment: Alignment.center,
      width: 50.toVWLength,
      height: 120.toPXLength,
      margin: DynamicEdgeInsets.symmetric(vertical: 10.toPXLength),
      backgroundDecoration: BoxDecoration(
          gradient:
              LinearGradient(colors: [Colors.cyanAccent, Colors.purpleAccent])),
    );

    endStyle = {
      ScreenScope(): Style(
          alignment: Alignment.center,
          width: 200.toPXLength,
          height: 100.toPXLength,
          margin: DynamicEdgeInsets.symmetric(vertical: 10.toPXLength),
          childAlignment: Alignment.center,
          shadows: [
            DynamicShapeShadow(
                blurRadius: 0.toPXLength,
                spreadRadius: 0.toPXLength,
                color: Colors.lightGreen,
                offset: DynamicOffset((0).toPXLength, (0).toPXLength)),
            DynamicShapeShadow(
                blurRadius: 5.toPXLength,
                spreadRadius: -5.toPXLength,
                color: Color.fromARGB(255, 240, 255, 240),
                offset: DynamicOffset((0).toPXLength, (0).toPXLength))
          ]),
    };
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
        leading: IconButton(
          icon: Icon(Icons.arrow_back),
          onPressed: () {
            Navigator.pop(context);
          },
        ),
      ),
      body: Center(
        child: ListView(
          children: [
            ConstrainedBox(
              constraints: BoxConstraints(
                maxWidth: 500,
                maxHeight: 500,
              ),
              child: AnimatedStyledContainer(
                duration: Duration(seconds: 3),
                style: toggleStyle ? beginStyle : endStyle,
                child: Container(),
              ),
            )
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.refresh),
        onPressed: () {
          setState(() {
            toggleStyle = !toggleStyle;
          });
        },
      ),
      // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}
